.*: Assembler messages:
.*:19: Error: branch to a symbol in another ISA mode
.*:21: Error: branch to a symbol in another ISA mode
.*:23: Error: branch to a symbol in another ISA mode
.*:25: Error: branch to a symbol in another ISA mode
