	.include "advsimd-armv8_3.s"

	fadd	v1.2d, v2.2d, v3.2d
	fadd	v1.2s, v2.2s, v3.2s
	fadd	v1.4s, v2.4s, v3.4s
	fadd	v0.4h, v0.4h, v0.4h
	fadd	v1.4h, v2.4h, v3.4h
	fadd	v0.8h, v0.8h, v0.8h
	fadd	v1.8h, v2.8h, v3.8h
